'Created by: X
'Web: http://www.createdbyx.com/
'Date: Monday, April 17, 2006
'=============================================

Public Class Form1
    Private Const NumberOfFlies As Integer = 25
    Private Const SpawnStinkyInterval As Integer = 1000 \ 30     ' 1000 ms div 30 fps
    Private Const UpdateFliesInterval As Integer = 1000 \ 60     ' 1000 ms div 60 fps

    Private mobjPlayer As Actor
    Private mobjFlies As Generic.List(Of Fly)
    Private mobjScents As Generic.List(Of Scent)
    Private WithEvents mobjTimer As Timers.Timer
    Private WithEvents mobjFlyUpdater As Timers.Timer
    Private WithEvents mobjStinkySpawner As Timers.Timer
    Private mobjGraphics As BufferedGraphics


    Public Sub DrawFlies()
        For Each F As Fly In mobjFlies
            DrawActor(F)
        Next
    End Sub

    Public Sub DrawReceptors()
        For Each F As Fly In mobjFlies
            For Each R As Receptor In F.Receptors
                Dim NX, NY As Single
                Displacement(F.Position.X, F.Position.Y, R.Distance, R.Direction + F.Direction, NX, NY)

                Dim Half As Single
                Half = R.Size / 2.0F
                mobjGraphics.Graphics.DrawLine(Pens.Yellow, F.Position.X, F.Position.Y, NX, NY)
                mobjGraphics.Graphics.DrawEllipse(Pens.Yellow, NX - Half, NY - Half, R.Size, R.Size)
            Next
        Next
    End Sub

    Public Sub DrawScents()
        For Each s As Scent In mobjScents
            Dim Half As Single
            Half = s.Strength / 2.0F
            mobjGraphics.Graphics.DrawEllipse(Pens.Green, s.Position.X - Half, s.Position.Y - Half, s.Strength, s.Strength)
        Next
    End Sub

    Public Sub DrawActor(ByVal A As Actor)
        Dim Half As Single
        Half = A.Size / 2.0F
        mobjGraphics.Graphics.DrawEllipse(Pens.Red, A.Position.X - Half, A.Position.Y - Half, A.Size, A.Size)
    End Sub


    Private Sub DoCleanUp()
        mobjTimer.Stop()
        mobjTimer = Nothing
        mobjStinkySpawner.Stop()
        mobjStinkySpawner = Nothing
        mobjFlyUpdater.Stop()
        mobjFlyUpdater = Nothing
        mobjScents.Clear()
        mobjScents = Nothing
        mobjPlayer = Nothing
        mobjFlies.Clear()
        mobjFlies = Nothing
        mobjGraphics.Dispose()
        mobjGraphics = Nothing
    End Sub

    Private Sub MakeStinky()
        mobjScents.Add(New Scent(mobjPlayer))
    End Sub

    Public Sub DecayFarts()
        Dim idx As Integer
        While idx <= mobjScents.Count - 1
            mobjScents(idx).Decay()
            If mobjScents(idx).Strength <= 0 Then
                mobjScents.RemoveAt(idx)
            Else
                idx += 1
            End If
        End While
    End Sub

    Private Sub KeepInBounds()
        ' keep the player within the visible area of the form
        mobjPlayer.Position.X = RestrictValue(mobjPlayer.Position.X, 0, Me.ClientSize.Width - 1)
        mobjPlayer.Position.Y = RestrictValue(mobjPlayer.Position.Y, 0, Me.ClientSize.Height - 1)

        ' keep all flies within the visible area of the form
        For Each F As Fly In mobjFlies
            F.Position.X = RestrictValue(F.Position.X, 0, Me.ClientSize.Width - 1)
            F.Position.Y = RestrictValue(F.Position.Y, 0, Me.ClientSize.Height - 1)
        Next
    End Sub

    Private Sub MoveFlies()
        Randomize(Now.Ticks)
        For Each F As Fly In mobjFlies
            Dim NX, NY As Single


            Displacement(F.Position.X, F.Position.Y, F.Speed, F.Direction, NX, NY)
            If NX > Me.ClientSize.Width - 1 Then F.Direction += CSng(Rnd() * Math.PI)
            If NX < 0 Then F.Direction += CSng(Rnd() * Math.PI)
            If NY > Me.ClientSize.Height - 1 Then F.Direction += CSng(Rnd() * Math.PI)
            If NY < 0 Then F.Direction += CSng(Rnd() * Math.PI)

            F.Position.X = CInt(NX)
            F.Position.Y = CInt(NY)
            Dim FoundScent As Boolean = False
            For Each R As Receptor In F.Receptors
                Displacement(F.Position.X, F.Position.Y, R.Distance, R.Direction + F.Direction, NX, NY)
                For Each S As Scent In mobjScents
                    If CircleCircle(S.Position.X, S.Position.Y, S.Strength, NX, NY, R.Distance) Then
                        F.Direction += R.Direction
                        ' Try using this line instead
                        ' F.Direction += ((R.Direction * 0.9F) + (Rnd() * (R.Direction * 0.2F)))
                        FoundScent = True
                        Exit For
                    End If
                Next
           Next
            If Not FoundScent Then F.Update()
        Next
    End Sub

    Private Sub Form_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Click
        Me.Close()
    End Sub

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs) Handles Me.MouseMove
        mobjPlayer.Position = New Point(e.X, e.Y)
    End Sub

    Private Sub Form_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        Me.Close()
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Not e.Cancel Then Me.DoCleanUp()
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' Resize form so that client size is 512x512
        Me.Size = New Size(512, 512) + (Me.Size - Me.ClientSize)

        ' create player object
        mobjPlayer = New Actor
        mobjPlayer.Position = New Point(Me.ClientSize.Width \ 2, Me.ClientSize.Height \ 2)
        mobjPlayer.Size = 10

        ' create flies and scent collections
        mobjFlies = New Generic.List(Of Fly)
        mobjScents = New Generic.List(Of Scent)

        ' add flies and receptors (Changing receptor values of one fly will change receptor of all flies)
        Dim R As New Generic.List(Of Receptor)
        R.Add(New Receptor(10, -(Math.PI / 4), 5))
        R.Add(New Receptor(10, Math.PI / 4, 5))
        Randomize(Now.Ticks)
        For idx As Integer = 0 To NumberOfFlies - 1
            Dim F As Fly
            F = New Fly(New Point(CInt(Rnd() * Me.ClientSize.Width), CInt(Rnd() * Me.ClientSize.Height)), R)
            F.Direction = CSng(Rnd() * (Math.PI * 2))
            mobjFlies.Add(F)
        Next

        ' Create graphics 
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserPaint, True)
        Drawing.BufferedGraphicsManager.Current.MaximumBuffer = New Size(1, 1) + Me.ClientSize
        mobjGraphics = Drawing.BufferedGraphicsManager.Current.Allocate(Me.CreateGraphics, Me.ClientRectangle)

        ' setup timers
        mobjTimer = New Timers.Timer
        mobjTimer.Interval = 1
        mobjTimer.AutoReset = False
        mobjTimer.Start()

        mobjStinkySpawner = New Timers.Timer
        mobjStinkySpawner.Interval = SpawnStinkyInterval
        mobjStinkySpawner.AutoReset = False
        mobjStinkySpawner.Start()

        mobjFlyUpdater = New Timers.Timer
        mobjFlyUpdater.Interval = UpdateFliesInterval
        mobjFlyUpdater.AutoReset = False
        mobjFlyUpdater.Start()
    End Sub

    Private Sub mobjTimer_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles mobjTimer.Elapsed
        If mobjGraphics Is Nothing Then Exit Sub

        Try
            mobjGraphics.Graphics.Clear(Color.Black)

            DrawFlies()
            DrawReceptors()
            DrawScents()
            DrawActor(mobjPlayer)

            KeepInBounds()
            DecayFarts()

            mobjGraphics.Graphics.DrawString("Click or press a key to exit...", Me.Font, Brushes.White, 50, 50)
            mobjGraphics.Render()
      
            mobjTimer.Start()
        Catch
        End Try
    End Sub

    Private Sub mobjStinkySpawner_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles mobjStinkySpawner.Elapsed
        If mobjGraphics Is Nothing Then Exit Sub

        Try
            MakeStinky()
            mobjStinkySpawner.Start()
        Catch
        End Try
    End Sub

    Private Sub mobjFlyUpdater_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles mobjFlyUpdater.Elapsed
        If mobjGraphics Is Nothing Then Exit Sub

        Try
            MoveFlies()
            mobjFlyUpdater.Start()
        Catch
        End Try
    End Sub
End Class
